
/* Generated data (by glib-mkenums) */

#ifndef GSD_IDENTITY_ENUM_TYPES_H
#define GSD_IDENTITY_ENUM_TYPES_H

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "gsd-smartcard-manager.h" */
GType gsd_smartcard_manager_error_get_type (void) G_GNUC_CONST;
#define GSD_TYPE_SMARTCARD_MANAGER_ERROR (gsd_smartcard_manager_error_get_type ())
G_END_DECLS

#endif /* GSD_IDENTITY_ENUM_TYPES_H */

/* Generated data ends here */

